package com.test.scaledemo;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.concurrent.TimeUnit;

import org.openspaces.admin.Admin;
import org.openspaces.admin.AdminFactory;
import org.openspaces.admin.gsa.GridServiceAgent;
import org.openspaces.admin.gsa.GridServiceAgents;
import org.openspaces.admin.gsa.GridServiceContainerOptions;
import org.openspaces.admin.gsc.GridServiceContainer;
import org.openspaces.admin.gsm.GridServiceManager;
import org.openspaces.admin.pu.ProcessingUnit;
import org.openspaces.admin.pu.ProcessingUnitInstance;
import org.openspaces.admin.pu.elastic.ElasticStatefulProcessingUnitDeployment;
import org.openspaces.admin.pu.elastic.config.ManualCapacityScaleConfigurer;
import org.openspaces.admin.space.ElasticSpaceDeployment;
import org.openspaces.core.util.MemoryUnit;

public class CopyOfScaleDemoMainOLD {
	static int MAX_MEMEORY_CAPACITY_GB=128;
	static int INITIAL_MEMEORY_CAPACITY_GB=16;
	static int TARGET_MEMEORY_CAPACITY_GB=64;

	static int MAX_CORES_CAPACITY=128;
	static int INITIAL_CORES_CAPACITY=64;
	static int TARGET_CORES_CAPACITY=32;
	
	public static void main(String[] args) throws Exception{

//		Admin admin = new AdminFactory().addLocator("10.10.10.249").create();
		Admin admin = new AdminFactory().create();
		admin.getGridServiceAgents().waitForAtLeastOne();
		
		GridServiceAgents agents = admin.getGridServiceAgents();
		for (GridServiceAgent gridServiceAgent : agents) {
			if (gridServiceAgent.getMachine().getHostName().equalsIgnoreCase("ride"))
			{
				GridServiceAgent agent = admin.getGridServiceAgents().waitForAtLeastOne();
				GridServiceContainerOptions gscOptions = new GridServiceContainerOptions();
				gscOptions.vmInputArgument("-Dcom.gs.zones=zone1,zone3");
				gscOptions.vmInputArgument("-Xmx256m");
				agent.startGridService(gscOptions);
			}
		}

		admin.getElasticServiceManagers().waitForAtLeastOne();
		GridServiceManager gsm = admin.getGridServiceManagers().waitForAtLeastOne();
		ProcessingUnit pu = null;
		
		pu = admin.getProcessingUnits().waitFor("mySpace", 5,TimeUnit.SECONDS);

		// check if the PU already running
		if (pu== null)
		{
			System.out.println(" PU not running - initial deploy");
			long startTime = System.currentTimeMillis();
			try
			{
				System.out.println(" Starting initial deploy - Deploying a PU with:" + TARGET_MEMEORY_CAPACITY_GB + "MB");
		//		pu = gsm.deploy(new ElasticStatefulProcessingUnitDeployment("myPU")	
				pu = gsm.deploy(new ElasticSpaceDeployment("mySpace")
		//		 .singleMachineDeployment()
				 .memoryCapacityPerContainer(4,MemoryUnit.GIGABYTES)
		         .maxMemoryCapacity(MAX_MEMEORY_CAPACITY_GB,MemoryUnit.GIGABYTES)
//		         .maxNumberOfCpuCores(MAX_CORES_CAPACITY)
				 .addContextProperty("cluster-config.groups.group.fail-over-policy.active-election.yield-time","300")
				 .addContextProperty("cluster-config.groups.group.fail-over-policy.active-election.fault-detector.invocation-delay","300")
				 .addContextProperty("cluster-config.groups.group.fail-over-policy.active-election.fault-detector.retry-count","2")
				 .addContextProperty("space-config.proxy-settings.connection-retries","5")
		         	//initial scale
		         	.scale(
		         			new ManualCapacityScaleConfigurer().
		         			memoryCapacity(INITIAL_MEMEORY_CAPACITY_GB,MemoryUnit.GIGABYTES).
		         			maxConcurrentRelocationsPerMachine(2).
//		         			numberOfCpuCores(INITIAL_CORES_CAPACITY).
		         			create())       
				);
//				System.out.println( "There are " + pu.getInstances().length + " running with " + pu.getNumberOfBackups()+ " backups");
				monitorPUScaleProgress(pu , INITIAL_MEMEORY_CAPACITY_GB);
				long endTime = System.currentTimeMillis();
				System.out.println(" Initial Deploy done! - Time to deploy system:" + (endTime-startTime)/1000 + " seconds");
			}
			catch (org.openspaces.admin.pu.ProcessingUnitAlreadyDeployedException e)
			{
				System.out.println(e.getMessage());
			}
		}
		
		// checking deployed PU.
		System.out.println(" About to start changing memory capacity from " + getPUTotalMemoryUtilization(pu)/1000 + " GB to " + TARGET_MEMEORY_CAPACITY_GB + "GB");
		System.out.println(" Hit enter to scale the data grid...");
		System.in.read();
		long startTime = System.currentTimeMillis();
		pu.scale(new ManualCapacityScaleConfigurer()
        .memoryCapacity(TARGET_MEMEORY_CAPACITY_GB,MemoryUnit.GIGABYTES)
//        .numberOfCpuCores(TARGET_CORES_CAPACITY)
        .maxConcurrentRelocationsPerMachine(2)
        .create());
		
		monitorPUScaleProgress(pu,TARGET_MEMEORY_CAPACITY_GB);
		long endTime = System.currentTimeMillis();
		System.out.println(" Memory capacity change done! - Time to scale system:" + (endTime-startTime)/1000 + " seconds");
		System.exit(0);
	}

	
	static void monitorPUScaleProgress(ProcessingUnit pu , int targetCapacity) throws Exception
	{
		double bias = targetCapacity/10; // 10 %
		
		while (true)
		{
			double currentMemUsageGB = getPUTotalMemoryUtilization(pu)/1000 ;
			Thread.sleep(2000);
			double diff = Math.abs (targetCapacity - currentMemUsageGB);
			int progressPercentage = (int)(100 - ((diff / targetCapacity) * 100));
			System.out.println(new Date() + ">> Total Memory used:"+
					currentMemUsageGB+ " GB - Progress:"+ (progressPercentage) + " % done");
			if (currentMemUsageGB> (targetCapacity - bias) 
					&& (currentMemUsageGB < (targetCapacity + bias)))
				break;
		}
	}
	
	static double getPUTotalMemoryUtilization(ProcessingUnit pu)
	{
		HashMap<String,Double> puJVMsSet = new HashMap<String,Double>();
//		System.out.println("Total Instance Count:" + pu.getNumberOfInstances() * 2);

		pu.waitFor(pu.getNumberOfInstances() * 2);
		double totalMemoryInMB=0;
		ProcessingUnitInstance instances[] = pu.getInstances();
		for (ProcessingUnitInstance processingUnitInstance : instances) {
			puJVMsSet.put(processingUnitInstance.getMachine().getHostAddress() + 
					processingUnitInstance.getVirtualMachine().getDetails().getPid(), 
					new Double (processingUnitInstance.getVirtualMachine().getDetails().
							getMemoryHeapMaxInMB()));
		}
//		System.out.println("Current total number of JVMs used:" + puJVMsSet.size());		
		Iterator<Double> iter = puJVMsSet.values().iterator();
		while (iter.hasNext())
		{
			totalMemoryInMB =totalMemoryInMB + iter.next().doubleValue(); 
		}
		return totalMemoryInMB;
	}	

}
