package com.gigaspaces.ucs.deploy;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

import org.openspaces.admin.Admin;
import org.openspaces.admin.AdminFactory;
import org.openspaces.admin.esm.ElasticServiceManager;
import org.openspaces.admin.esm.deployment.ElasticDataGridDeployment;
import org.openspaces.admin.pu.ProcessingUnit;
import org.openspaces.admin.space.Space;
import org.openspaces.core.GigaSpace;
import org.openspaces.grid.esm.ElasticScaleHandlerConfig;

import com.gigaspaces.ucs.deploy.AgentlessInstaller.InstallationDetails;
import com.j_spaces.core.client.SQLQuery;

/*******
 * A demonstration of the ESM managed data grid functionality.
 * 
 * @author barakme
 * 
 */
public final class ESMDemo {

	/****
	 * Tester for the Gigaspaces JDBC functionality.
	 */
	public static final class JDBCQuery {
		/****
		 * Runs the JDBC test.
		 * 
		 * @param locator
		 *            Locator for the Lookup Service.
		 * @throws Exception
		 *             if the test failed.
		 */
		public static void testJDBC(final String locator) throws Exception {

			Connection conn;
			Class.forName("com.j_spaces.jdbc.driver.GDriver").newInstance();
			final String url = "jdbc:gigaspaces:url:jini://*/*/myElasticDataGrid?locators=" + locator;
			conn = DriverManager.getConnection(url);
			final Statement st = conn.createStatement();
			final String query = "SELECT * FROM com.gigaspaces.ucs.deploy.Data";
			final ResultSet rs = st.executeQuery(query);

			// Iterate through the result set
			int i = 0;
			while (rs.next()) {
				// String s = rs.getString("COF_NAME");
				// float n = rs.getFloat("PRICE");
				System.out.println("Data [" + (i++) + "] " + rs.getString("data"));// +
				// " salary="
				// +
				// rs.getObject("info.salary"));
			}

		}

		private JDBCQuery() {

		}

	}

	private static final String PROPERTIES_FILE = "ucs.properties";

	/********
	 * Create the test data.
	 * 
	 * @param i
	 *            index.
	 * @return the test data.
	 */
	public static Map<String, Object> createInfo(final long i) {
		final Map<String, Object> info = new HashMap<String, Object>();
		info.put("address", i + " Broadway");
		info.put("socialSecurity", 1232287642L + i);
		info.put("salary", 10000 + i);
		return info;
	}

	private static InstallationDetails createInstallationDetails(final Properties props, final String targetIp) {
		final InstallationDetails details = new InstallationDetails();

		details.setLocalDir(props.getProperty("localDir"));
		details.setRemoteDir(props.getProperty("remoteDir"));
		details.setUsername(props.getProperty("SSHUsername"));
		details.setPassword(props.getProperty("SSHPassword"));

		details.setLocator(targetIp);
		details.setTargetIP(targetIp);
		details.setLus(true);

		return details;
	}

	private static Properties loadProperties() throws IOException {
		final Properties props = new Properties();
		final File file = new File(PROPERTIES_FILE);
		if (!file.exists()) {
			props.put("user", "MY_UCS_MANAGER_USERNAME");
			props.put("password", "MY_UCS_MANAGER_PASSWORD");
			props.store(new FileWriter(file), "Generated properties file for ESM Demo UCS properties");
			throw new FileNotFoundException("Could not find properties file: " + file.getAbsolutePath()
					+ ". A default one was created, " + "please set the values according to your account");
		}

		final Reader reader = new FileReader(file);
		try {
			props.load(reader);
			return props;
		} finally {
			reader.close();
		}

	}

	/**
	 * The main method for the demo.
	 * 
	 * @param args
	 *            not used.
	 * @throws Exception .
	 */
	public static void main(final String[] args) throws Exception {

		final Properties props = ESMDemo.loadProperties();

		final String locator = "10.122.170.99";
		final InstallationDetails details = ESMDemo.createInstallationDetails(props, locator);

		final AgentlessInstaller installer = new AgentlessInstaller();

		// Creating a data grid on demand
		System.out.println("Deploying Data Grid..");

		// Launch the head node, with the GigaSpaces Elastic Scaling Module
		ESMDemo.startESMNode(installer, details);

		// Starts the GS-UI with the locator
		// startGUI(locator);

		// Initialize the GigaSpaces Admin API with the locator for the head
		// node
		final Admin admin = new AdminFactory().addLocator(locator).createAdmin();

		final String gridName = "myElasticDataGrid";
		// Use the Admin API to deploy an elastic grid
		final ElasticServiceManager esm = admin.getElasticServiceManagers().waitForAtLeastOne();
		final ProcessingUnit pu = esm.deploy(new ElasticDataGridDeployment(gridName).elasticScaleHandler(
				new ElasticScaleHandlerConfig("com.gigaspaces.ucs.scaleHandler.UCSElasticScaleHandler")

				.addProperty("Uri", props.getProperty("UCS_URI")).addProperty("UCSUsername",
						props.getProperty("UCSUsername")).addProperty("UCSPassword", props.getProperty("UCSPassword"))
						.addProperty("SSHUsername", props.getProperty("SSHUsername")).addProperty("SSHPassword",
								props.getProperty("SSHPassword")).addProperty("locator", locator).addProperty(
								"lookupGroup", props.getProperty("lookupGroup")).addProperty("gridName", gridName)

						.addProperty("BladePoolName", props.getProperty("BladePoolName")).addProperty("macPoolName",
								props.getProperty("macPoolName"))
						.addProperty("Template", props.getProperty("Template")).addProperty("Organization",
								props.getProperty("Organization")).addProperty("serverNamePrefix",
								props.getProperty("serverNamePrefix")).addProperty("localDir",
								props.getProperty("remoteDir"))
						.addProperty("remoteDir", props.getProperty("remoteDir"))

		).capacity("1G", "2G").highlyAvailable(true).publicDeploymentIsolation().maximumJavaHeapSize("250m")
				.initialJavaHeapSize("250m"));

		try {

			System.out.println("Waiting for data grid deployment completion ..");
			final Space space = pu.waitForSpace();

			while (!space.waitFor(space.getTotalNumberOfInstances(), 10, TimeUnit.SECONDS)) {
				System.out.println("Waiting for all partitions to deploy. " + "Available: "
						+ space.getNumberOfInstances() + ", " + "Required: " + space.getTotalNumberOfInstances());
			}

			// Write some data...
			final GigaSpace gigaSpace = space.getGigaSpace();

			System.out.println("Writing 1000 objects");
			for (long i = 0; i < 1000; i++) {
				gigaSpace.write(new Data(i, "message" + i, ESMDemo.createInfo(i)));
			}

			System.out.println("Reading 1000 objects");
			final Data[] d = gigaSpace.readMultiple(new SQLQuery<Data>(Data.class,
					"info.salary < 11000 and info.salary >= 10000"), Integer.MAX_VALUE);
			System.out.println("SQLQuery<Data>(Data.class, info.salary < 15000 and info.salary >= 8000) results -> ["
					+ d.length + " ]");

			System.out.println("Reading 1000 using JDBC connector objects");

			JDBCQuery.testJDBC(locator);
			// System.out.println("Clearing...");
			// pu.undeploy();
			System.out.println("Done!");
			System.exit(0);

		} catch (final Exception ex) {
			// for debugging purposes clean the deployment in case of exception
			ex.printStackTrace();
			System.out.println("Caught exception: Undeploying... ");
			pu.undeploy();
			System.exit(1);
		}
	}

	private static void startESMNode(final AgentlessInstaller installer, final InstallationDetails details)
			throws IOException {

		if (installer.checkConnection(details.getTargetIP(), 4164, 3)) {
			return;
		}

		installer.installOnMachineWithIP(details);
	}

	/******
	 * Use this to launch the GS UI.
	 * 
	 * @param locator
	 *            .
	 * @throws IOException .
	 */
	/*private static void startGUI(final String locator) throws IOException {
		final ProcessBuilder pb = new ProcessBuilder(new String[] { "cmd.exe", "/c", "start",
				"c:/Gigaspaces/gigaspaces-xap-premium-7.1.1-ga/bin/gs-ui.bat" });
		pb.environment().put("LOOKUPLOCATORS", locator);
		pb.start();

	}*/

	private ESMDemo() {

	}

}
