package com.gigaspaces.rackspace.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import com.sun.xml.internal.messaging.saaj.util.ByteOutputStream;

public final class FilesUtils {
	/*******
	 * Private constructor to avoid instantiation of utility class.
	 */
	private FilesUtils() {
		// ignore.
	}

	public static byte[] getBytesFromFile(final File file) throws IOException {
		final InputStream is = new FileInputStream(file);
		final byte[] bytes = FilesUtils.getBytesFromStream(is, file.length());
		return bytes;
	}

	public static byte[] getBytesFromResource(final String resource) throws IOException {

		final File file = new File(resource);
		if (file.exists()) {
			return FilesUtils.getBytesFromFile(file);
		}

		final InputStream input = Thread.currentThread().getContextClassLoader()
				.getResourceAsStream(resource);
		if (input == null) {
			throw new IllegalArgumentException("Could not find resource: " + resource);
		}
		
		ByteOutputStream output = null;
		try {
			output = new ByteOutputStream();
			output.write(input);
			return output.getBytes();
		} finally {
			if (output != null) {
				output.close();
			}
		}

	}

	private static byte[] getBytesFromStream(final InputStream is, final long length)
			throws IOException {

		if (length > Integer.MAX_VALUE) {
			throw new IOException("Can't read Resource: Too long");
		}

		final byte[] bytes = new byte[(int) length];

		int offset = 0;
		int numRead = 0;
		while ((offset < bytes.length)
				&& ((numRead = is.read(bytes, offset, bytes.length - offset)) >= 0)) {
			offset += numRead;
		}

		if (offset < bytes.length) {
			throw new IOException("Could not completely read resource");
		}

		is.close();
		return bytes;
	}

	public static String getFileFromResource(final String fileName) {

		final Thread currentThread = Thread.currentThread();
		final ClassLoader classLoader = currentThread.getContextClassLoader();
		final URL url = classLoader.getResource(fileName);
		return url.getFile();

	}

}
